! Program DrawArcs Draws a set of a rocker arcs, with marked lengths and radii,
! and angle measures, in Postscript. Used to measure figure skating blades.
 
! Written by Mitchell Grunes.
! This is a free format fortran 90 program.
! Cygwin or linux compiles this using
!  gfortran -Wall DrawArcs.f90 -o DrawArcs
 
! Cygwin produces executable DrawArcs.exe; Linux produces DrawArcs.
 
! Postscript language reference:
!  https://www.adobe.com/jp/print/postscript/pdfs/PLRM.pdf
! See also https://www.adobe.chttps://personal.math.ubc.ca/~cass/graphics/text/old.pdf/last/ch4.pdfom/jp/print/postscript/pdfs/PLRM.pdf
!  https://helpx.adobe.com/x-productkb/global/troubleshoot-postscript-errors.html
!  https://personal.math.ubc.ca/~cass/graphics/text/old.pdf/last/ch4.pdf
 
! Can convert postscript to PDF using https://cloudconvert.com/ps-to-pdf.
! Or https://online2pdf.com/convert-ps-to-pdf seems able to take larger size pages.
 
! Can possibly convert postscript to Cad format using a free Cad
!  program, like DoubleCad XT - download from
!  https://www.turbocad.com/content/doublecad-xt-v5
!  Instructions are at
!  https://www.youtube.com/watch?v=RDd1M9Zvl44
! Or FreeCAD.https://wiki.freecad.org/Import_Export
!  Says it can import PDF.
! Or LibreCad? https://librecad.org
!  Doesn't appear able to import postscript or pdf.
! Or LinuxCAD?
! And search google for "Free 2D CAD software like AutoCAD"
! Or maybe use http://www.calvina.de/pstoedit to convert postscript to DXF?
! https://sourceforge.net/projects/thandxflib/postdownload describes a fortran
!  library to write DXF.
! https://sourceforge.net/projects/thandxflib/files/dxflib/dxflib-1.0.0.zip/download
 
! https://h30434.www3.hp.com/t5/LaserJet-Printing/print-postscript-file/td-p/5594534
! has instructions on using lpr to print postscript files
 
! Perhaps I should use
! https://opensource.adobe.com/dc-acrobat-sdk-docs/pdfstandards/PDF32000_2008.pdf
! to instead write PDF files?
! Or
! https://www.opendesign.com/files/guestdownloads/OpenDesign_Specification_for_.dwg_files.pdf
! to create a DWG file?
 
common WidLin,io
character*100 ParmFile,PsFile           ! Name of BladeDraw & Postscript files.
character*10 RockerFmt                  ! Room to display rocker in xx.x or x.xx with
                                        !  g0.3 format
print*,'Enter 1 to input from arc parameter file, 0 to make a new one:'
io=1
read*,io
if(io.eq.0)print*,'Arc parameter file (Enter blank for "DrawArcs.txt") to write:'
if(io.ne.0)print*,'Arc parameter file (Enter blank for "DrawArcs.txt") to read:'
read(*,'(a)')ParmFile
if(ParmFile.eq.' ')ParmFile='DrawArcs.txt'
print*,'ParmFile=',ParmFile
if(io.eq.0)open (1,file=ParmFile,status='new',err=1)
if(io.ne.0)open (1,file=ParmFile,status='old')
goto 2
1 print*,'File already exists - overwrite(0=no,1=yes?):'
  i=0
  read*,i
  if(i.eq.0)stop 'Cannot continue'
  if(io.eq.0)open (1,file=ParmFile,status='unknown')
 2 print*,'Output postscript file (Enter blank for "DrawArcs.ps"):'
if(io.eq.0)then
  read(*,'(a)')PsFile
  if(PsFile.eq.' ')PSFile='DrawArcs.ps'
  write(1,'(a)')PSFile
else
  read(1,'(a)')PSFile
  if(PsFile.eq.' ')PSFile='DrawArcs.ps'
endif
 
print*,'PsFile=',PSFile
open (2,file=PsFile,status='unknown')
 
write(2,'(a)')'%!PS'                    ! So print spools detect start of file
write(2,'(a,g0,1x,g0,a)')'<< /PageSize [',8.5*72,11*72,'>> setpagedevice'
print   '(a)','% *** This plot is designed for 8.5 x 11 inch Paper.***'
write(2,'(a)')'% *** This plot is designed for 8.5 x 11 inch Paper.***'
write(2,'(a)')'% ***Rocker Arcs & angle measures created by Mitch Grunes''s DrawArcs***'
 
print*,'***On Mitch''s printer, following needs to be 9/8=1.125***!'
call ReadFlt('X scaling factor to make inches exact(1.?)',xScale,1.)
print*,'***On Mitch''s printer, following needs to be 12/11=1.090909091***!'
call ReadFlt('Y scaling factor to make inches exact(1.?)',yScale,1.)
write(2,'(g0,1x,g0,a)')72*xScale,72*yscale,' scale'

call ReadFlt('Line width (inches) (.01?)',WidLin,.01)
write(2,'(g0,a)')WidLin,' setlinewidth'  ! Graphics line width in 1/72 inch (point) units.
write(2,'(a)') '/Times-Bold findfont .1 scalefont setfont'  
write(2,'(a)') '/ms {moveto show} bind def  % move to position and show text'
 
call WriteBlank
 
write(2,'(a)')'1 setlinecap'            ! Set shape of line ends for stroke
                                        ! (0=butt,1=round,2=square).
write(2,'(a)')'1 setlinejoin'           ! Set shape of corners for stroke
                                        ! (0=miter,1=round,2=bevel).
 
call ReadFlt('Margin Size(.55?)',TheMargin,.55) !Will print on 8.5x11 inch paper.
x1=TheMargin
x2=8.5-TheMargin
y1=TheMargin
y2=11-TheMargin
call WriteBlank
write(2,'(a)') '% ***Box around area, and centerline***'
call DrawSeg(x1,y1,x2,y1)               ! Draw box around edges.
call DrawSeg(x1,y2,x2,y2)
call DrawSeg(x1,y1,x1,y2)
call DrawSeg(x2,y1,x2,y2)
xC=(x1+x2)/2                            ! Central position
yC=(y1+y2)/2                            ! Used for centerline
call DrawSeg(x1,yC,x2,yC)
call ReadInt('# of rocker curves/inch(8?)',idInch,8)
dInch=1./idInch                        ! Inches between rocker curves
!call ReadFlt('Tic Length (inches)(.2?)',TicLen,.2)
TicLen=.8*dInch
 
                                        ! This title is outside margins
write(2,'(1h(,a,2h) ,g0,1x,g0,a)') 'Rocker Arcs (inches)',(x1+x2/2-.8),y2+TicLen, ' ms'
 
! Draw y scale to check print scaling.
write(2,'(a)') '% ***Tics along y axis***'
dy=(y2-y1)/2
!do y=0,dy,dInch/2                       ! Twice as many tics as rocker curves
do i=0,ifix(dy/dInch*2)
  TicLen2=TicLen/2                                   ! Most tics are small
  if(i/(idInch  )*(idInch  ).eq.i)TicLen2=TicLen*.75 ! Every 1/2 inch are medium
  if(i/(idInch*2)*(idInch*2).eq.i)TicLen2=TicLen     ! Every inch are large
  y=i*dInch/2
  call DrawSeg(x1,yC+y, x1-TicLen2,yC+y) ! Above & below centerline
  call DrawSeg(x1,yC-y, x1-TicLen2,yC-y)
  call DrawSeg(x2,yC+y, x2+TicLen2,yC+y) ! And on right side
  call DrawSeg(x2,yC-y, x2+TicLen2,yC-y)
enddo

! Draw x scale on top, bottom, and below Centerline.
write(2,'(a)') '% ***Tics along x axis***'
dx=(x2-x1)
!do x=0,dx,dInch/2
do i=0,ifix(dx/dInch*2)
  TicLen2=TicLen/2                                   
  if(i/(idInch  )*(idInch  ).eq.i)TicLen2=TicLen*.75 
  if(i/(idInch*2)*(idInch*2).eq.i)TicLen2=TicLen     
  x=x1+i*dInch/2
  call DrawSeg(x,y1,   x,y1-TicLen2)     ! On bottom side
  call DrawSeg(x,y2,   x,y2+TicLen2)     ! And on top side
  call DrawSeg(x,yC,   x,yC-TicLen2)     ! And on centerline
enddo
 
iter=1                                   ! First time through
10 if(iter.ne.1)print*,'***To stop program, input following as 0***'
if(iter.ne.1)print*,'***Value cannot be larger than',RLo,'***'
if(iter.eq.1)call ReadFlt('Highest rocker length(99?)',RHi,99.)
if(iter.eq.2)call ReadFlt('Highest rocker length(30?)',RHi,30.)
if(iter.gt.2)call ReadFlt('Highest rocker length(0?)',RHi,0.)
if(RHi.eq.0)goto 999
if(iter.eq.1)call ReadFlt('Lowest rocker length(78?)',RLo,78.)
if(iter.ge.2)call ReadFlt('Lowest rocker length(0?)', RLo,6.)
if(iter.eq.1)call ReadFlt('Rocker Length Decrement(3?)',dR,3.)
if(iter.ne.1)call ReadFlt('Rocker Length Decrement(1?)',dR,1.)

if(iter.eq.1)xLo=TheMargin
nRocker=(Rhi-RLo)/dR+1.001
do iRocker=0,nRocker-1
  R=RHi-iRocker*dR
  xLo=xLo+dInch
  xCenter=xLo+R
  yCenter=yC
  dy=(y2-y1)/2
  if(R.le.dy-.01)goto 998                 ! Radius not large enough to hit top or bottom
  theta=asind(dy/R) ! Angle to top line from center.
  if(theta.gt.90)goto 998                 ! Angles over 90 from centerline mess up math.
  theta1=180-theta                        ! Angles to reach top & bottom line.
  theta2=180+theta
  xStart=xCenter+R*cosd(theta1)           ! X where arc reaches top or bottom line.
  if(xStart.gt.x2)goto 998                ! Don't let ends of arc go outside box
  print   '(a,g0,a)', '***Rocker=',R,' inches ***'
  write(2,'(a,g0,a)') '% ***Rocker=',R,' inches***'
  call DrawArc(xCenter,yCenter,R,theta1,theta2)
  RockerFmt=' '
  !print '(a,1h",g0.3,1h")', 'R=',R
  write(RockerFmt,'(g0.3)') R
  i=10
  do while(RockerFmt(i:i).eq.' '.or.RockerFmt(i:i).eq.'.')
    i=i-1
  enddo
  if(i.gt.2)then
    if(RockerFmt(i-1:i).eq.'.0')i=i-2
  endif
  if(i.gt.3)then
    if(RockerFmt(i-2:i).eq.'.00')i=i-3
  endif
  write(2,'(1h(,a,2h) ,g0,1x,g0,a)') RockerFmt(1:i),xStart -i*.05,y2-.08, ' ms'
  write(2,'(1h(,a,2h) ,g0,1x,g0,a)') RockerFmt(1:i),xStart -i*.05,y1+.01, ' ms'
  write(2,'(1h(,a,2h) ,g0,1x,g0,a)') RockerFmt(1:i),xLo    -i*.05,yC+.01,' ms'
  if(iter.eq.1.and.iRocker.eq.0)then                 ! Label straight line as infinity.
    write(2,'(1h(,a,2h) ,g0,1x,g0,a)') "infinity",xLo-dInch-.32,y2-.08, ' ms'
    write(2,'(1h(,a,2h) ,g0,1x,g0,a)') "infinity",xLo-dInch-.32,y1+.01, ' ms'
    write(2,'(1h(,a,2h) ,g0,1x,g0,a)') "infinity",xLo-dInch-.32,yC+.01,' ms'
  endif
  write(2,'(a)') '% ***Drawing Scale along that arc***'
  do i=1,10000                         ! Draw Scales along curved arcs
    theta=(dInch*i/2)/R*180/3.1415926535897932 ! Arc length from 180 deg
 
    x   =xCenter+R*cosd(180-theta)
    y   =yCenter+R*sind(180-theta)
    if(y.gt.y2)goto 99                 ! Stop scales at box edges.
    TicLen2=TicLen/2                                   
    if(i/(idInch  )*(idInch  ).eq.i)TicLen2=TicLen*.75 
    if(i/(idInch*2)*(idInch*2).eq.i)TicLen2=TicLen     

    xEnd=xCenter+(R+TicLen2)*cosd(180-theta)
    yEnd=yCenter+(R+TicLen2)*sind(180-theta)
    call DrawSeg(x,y,xEnd,yEnd)
 
    x   =xCenter+R*cosd(180+theta)
    y   =yCenter+R*sind(180+theta)
    xEnd=xCenter+(R+TicLen2)*cosd(180+theta)
    yEnd=yCenter+(R+TicLen2)*sind(180+theta)
    call DrawSeg(x,y,xEnd,yEnd)
  enddo
99 enddo

iter=iter+1
call WriteBlank
goto 10
 
998 print'(a,g0,a)', '***Could not draw R=',R,' Curve***'
999 call WriteBlank
write(2,'(a)')'% ***Finished!***'
write(2,'(a)')'showpage'                ! prints the prepared page.
close(1)
close(2)
print*,'Ghostview can display this, e.g.,'
print*,' gv -media=Letter DrawArcs.ps'
stop 'DrawArcs completed successfuly.'
end
! --------------------------------------------------------------------------------------
subroutine ReadInt(Prompt,num,iDefault) ! Read prompted integer, echo to parameter
                                        ! File, or to screen if from file.
common WidLin,io
character*(*) Prompt
1 num=iDefault
if(io.eq.0)then
  print'(a)',prompt//':'
  print'(a,i0,a)','Enter / for default=',iDefault,')'
  read*,num
  write(1,'(i17,6x,a)')num,prompt
else
  print'(a,$)',prompt//':'              ! Print prompt first, in case of i/o error.
  read(1,*,end=99)num
  print'(i0)',num
endif
return
99 print*,'*****Parameter file too short - continue editing from keyboard!*****'
io=0
open(1,access='append')
goto 1
end
! --------------------------------------------------------------------------------------
subroutine WriteBlank                   ! Prints blank line on screen, parameter
                                        ! file (if io.eq.0), and postscript.
common WidLin,io
print*,' '
if(io.eq.0)write(1,*)' '
write(2,*)' '
return
end
! --------------------------------------------------------------------------------------
subroutine Readflt(Prompt,val,Default)   ! Similar for float point
common WidLin,io
character*(*) Prompt
1 val=Default
if(io.eq.0)then
  print'(a)',prompt//':'
  print'(a,g0,a)','(Enter / for default=',Default,')'
  val=Default
  read*,val
  write(1,'(g21.9,2x,a)')val,prompt
else
  print'(a,$)',prompt//':'
  read(1,*,end=99)val
  print'(g0)',val
endif
return
99 print*,'*****Parameter file too short - continue editing from keyboard!*****'
io=0
open(1,access='append')
goto 1
end
! --------------------------------------------------------------------------------------
                                         ! Draw CCW circular arc to path.
subroutine DrawArc(xCenter,yCenter,R,theta1,theta2)
common WidLin,io
!       Input parameters:
! xCenter,yCenter=center of circle
! R=radius of circle
! theta1,theta2=Angle range. theta1.lt.theta2, and both must be in [-180 to 0] deg.
!       Modified parameters:
write(2,'(a)') 'newpath'
!print '(5(g0,1x),a)',  xCenter,yCenter,R,theta1,theta2,'arc'
write(2,'(5(g0,1x),a)')xCenter,yCenter,R,theta1,theta2,'arc'
write(2,'(a)') 'stroke'
 
return
end
! --------------------------------------------------------------------------------------
subroutine DrawSeg(x1,y1,x2,y2)           ! Draw line segment outside path.
!print       '(a,g0,1x,g0,a,g0,1x,g0,a)', &
!& 'newpath ',x1,y1,' moveto ',x2,y2,' lineto stroke'
write(2,'(a,g0,1x,g0,a,g0,1x,g0,a)') &
 & 'newpath ',x1,y1,' moveto ',x2,y2,' lineto stroke'
end
