! Program DrawGrid Draws a .5 inch grid, with 1/10 inch tics, to test printer 
!  x,y scaling and
! whether they are at right angles, in Postscript. 
 
! Written by Mitchell Grunes.
! This is a free format fortran 90 program.
! Cygwin or linux compiles this using
!  gfortran -Wall DrawGrid.f90 -o DrawGrid
 
! Cygwin produces executable DrawGrid.exe; Linux produces DrawGrid.
 
! Postscript language reference:
!  https://www.adobe.com/jp/print/postscript/pdfs/PLRM.pdf
! See also https://www.adobe.chttps://personal.math.ubc.ca/~cass/graphics/text/old.pdf/last/ch4.pdfom/jp/print/postscript/pdfs/PLRM.pdf
!  https://helpx.adobe.com/x-productkb/global/troubleshoot-postscript-errors.html
!  https://personal.math.ubc.ca/~cass/graphics/text/old.pdf/last/ch4.pdf
 
! Can convert postscript to PDF using https://cloudconvert.com/ps-to-pdf.
! Or https://online2pdf.com/convert-ps-to-pdf seems able to take larger size pages.
 
! Can possibly convert postscript to Cad format using a free Cad
!  program, like DoubleCad XT - download from
!  https://www.turbocad.com/content/doublecad-xt-v5
!  Instructions are at
!  https://www.youtube.com/watch?v=RDd1M9Zvl44
! Or FreeCAD.https://wiki.freecad.org/Import_Export
!  Says it can import PDF.
! Or LibreCad? https://librecad.org
!  Doesn't appear able to import postscript or pdf.
! Or LinuxCAD?
! And search google for "Free 2D CAD software like AutoCAD"
! Or maybe use http://www.calvina.de/pstoedit to convert postscript to DXF?
! https://sourceforge.net/projects/thandxflib/postdownload describes a fortran
!  library to write DXF.
! https://sourceforge.net/projects/thandxflib/files/dxflib/dxflib-1.0.0.zip/download
 
! https://h30434.www3.hp.com/t5/LaserJet-Printing/print-postscript-file/td-p/5594534
! has instructions on using lpr to print postscript files
 
! Perhaps I should use
! https://opensource.adobe.com/dc-acrobat-sdk-docs/pdfstandards/PDF32000_2008.pdf
! to instead write PDF files?
! Or
! https://www.opendesign.com/files/guestdownloads/OpenDesign_Specification_for_.dwg_files.pdf
! to create a DWG file?
 
common WidLin,io
character*100 ParmFile,PsFile           ! Name of BladeDraw & Postscript files.
print*,'Enter 1 to input from arc parameter file, 0 to make a new one:'
io=1
read*,io
if(io.eq.0)print*,'Arc parameter file (Enter blank for "DrawGrid.txt") to write:'
if(io.ne.0)print*,'Arc parameter file (Enter blank for "DrawGrid.txt") to read:'
read(*,'(a)')ParmFile
if(ParmFile.eq.' ')ParmFile='DrawGrid.txt'
print*,'ParmFile=',ParmFile
if(io.eq.0)open (1,file=ParmFile,status='new',err=1)
if(io.ne.0)open (1,file=ParmFile,status='old')
goto 2
1 print*,'File already exists - overwrite(0=no,1=yes?):'
  i=0
  read*,i
  if(i.eq.0)stop 'Cannot continue'
  if(io.eq.0)open (1,file=ParmFile,status='unknown')
 2 print*,'Output postscript file (Enter blank for "DrawGrid.ps"):'
if(io.eq.0)then
  read(*,'(a)')PsFile
  if(PsFile.eq.' ')PSFile='DrawGrid.ps'
  write(1,'(a)')PSFile
else
  read(1,'(a)')PSFile
  if(PsFile.eq.' ')PSFile='DrawGrid.ps'
endif
 
print*,'PsFile=',PSFile
open (2,file=PsFile,status='unknown')
 
write(2,'(a)')'%!PS'                    ! So print spools detect start of file
write(2,'(a,g0,1x,g0,a)')'<< /PageSize [',8.5*72,11*72,'>> setpagedevice'
print   '(a)','% *** This plot is designed for 8.5 x 11 inch Paper.***'
write(2,'(a)')'% *** This plot is designed for 8.5 x 11 inch Paper.***'
write(2,'(a)')'% ***Grid created by Mitch Grunes''s DrawGrid***'
 
print*,'***On Mitch''s printer, following needs to be 9/8=1.125***!'
call ReadFlt('X scaling factor to make inches exact(1.?)',xScale,1.)
print*,'***On Mitch''s printer, following needs to be 12/11=1.090909091***!'
call ReadFlt('Y scaling factor to make inches exact(1.?)',yScale,1.)
write(2,'(g0,1x,g0,a)')72*xScale,72*yscale,' scale'

call ReadFlt('Line width (inches) (.01?)',WidLin,.01)
write(2,'(g0,a)')WidLin,' setlinewidth'  ! Graphics line width in 1/72 inch (point) units.
 
call WriteBlank
 
write(2,'(a)')'1 setlinecap'            ! Set shape of line ends for stroke
                                        ! (0=butt,1=round,2=square).
write(2,'(a)')'1 setlinejoin'           ! Set shape of corners for stroke
                                        ! (0=miter,1=round,2=bevel).
 
call ReadFlt('Margin Size(.55?)',TheMargin,.55) !Will print on 8.5x11 inch paper.
x1=TheMargin
x2=8.5-TheMargin
y1=TheMargin
y2=11-TheMargin
call WriteBlank
do x=x1,x2+.001,.5                      ! Yes I know - real do loops=deprecated feature.
  call DrawSeg(x,y1,x,y2)               ! Horizontal lines
enddo
Call WriteBlank
do y=y1,y2+.001,.5
  call DrawSeg(x1,y,x2,y)               ! Vertical lines
enddo
call WriteBlank
do x=x1,x2+.001,.1                      ! Yes I know - real do loops=deprecated feature.
  call DrawSeg(x,y1,x,y1+.1)            ! Horizontal tics
enddo
Call WriteBlank
do y=y1,y2+.001,.1
  call DrawSeg(x1,y,x1+.1,y)            ! Vertical tics
enddo
call WriteBlank
write(2,'(a)')'% ***Finished!***'
write(2,'(a)')'showpage'                ! prints the prepared page.
close(1)
close(2)
print*,'Ghostview can display this, e.g.,'
print*,' gv -media=Letter DrawGrid.ps'
stop 'DrawGrid completed successfuly.'
end
! --------------------------------------------------------------------------------------
subroutine ReadInt(Prompt,num,iDefault) ! Read prompted integer, echo to parameter
                                        ! File, or to screen if from file.
common WidLin,io
character*(*) Prompt
1 num=iDefault
if(io.eq.0)then
  print'(a)',prompt//':'
  print'(a,i0,a)','Enter / for default=',iDefault,')'
  read*,num
  write(1,'(i17,6x,a)')num,prompt
else
  print'(a,$)',prompt//':'              ! Print prompt first, in case of i/o error.
  read(1,*,end=99)num
  print'(i0)',num
endif
return
99 print*,'*****Parameter file too short - continue editing from keyboard!*****'
io=0
open(1,access='append')
goto 1
end
! --------------------------------------------------------------------------------------
subroutine WriteBlank                   ! Prints blank line on screen, parameter
                                        ! file (if io.eq.0), and postscript.
common WidLin,io
print*,' '
if(io.eq.0)write(1,*)' '
write(2,*)' '
return
end
! --------------------------------------------------------------------------------------
subroutine Readflt(Prompt,val,Default)   ! Similar for float point
common WidLin,io
character*(*) Prompt
1 val=Default
if(io.eq.0)then
  print'(a)',prompt//':'
  print'(a,g0,a)','(Enter / for default=',Default,')'
  val=Default
  read*,val
  write(1,'(g21.9,2x,a)')val,prompt
else
  print'(a,$)',prompt//':'
  read(1,*,end=99)val
  print'(g0)',val
endif
return
99 print*,'*****Parameter file too short - continue editing from keyboard!*****'
io=0
open(1,access='append')
goto 1
end
! --------------------------------------------------------------------------------------
subroutine DrawSeg(x1,y1,x2,y2)           ! Draw line segment outside path.
!print       '(a,g0,1x,g0,a,g0,1x,g0,a)', &
!& 'newpath ',x1,y1,' moveto ',x2,y2,' lineto stroke'
write(2,'(a,g0,1x,g0,a,g0,1x,g0,a)') &
 & 'newpath ',x1,y1,' moveto ',x2,y2,' lineto stroke'
end
